c chi2.f
c Calculate chi^2 for given orbit of xi Tau.
c Miroslav Broz (miroslav.broz@email.cz), Jul 20th 2015

      program chi2_

      implicit none
      include '../simplex/simplex.inc'
      include '../simplex/dependent.inc'

      integer i
      real*8 x(NDIMMAX),chi2,probp,probq,nu
      character*80 str

      integer length
      real*8 chi2_func,gammp,gammq

c  read input parameters
5     continue
        read(*,10,err=990,end=990) str
10      format(a)
      if (str(1:1).eq.'#') goto 5

      read(str,*,err=990,end=990) nparam
      if (nparam.gt.NDIMMAX) then
        write(*,*) 'Error: nparam > NDIMMAX = ', NDIMMAX
        stop
      endif

      read(*,*,err=990,end=990) (x_param(i), i=1,nparam)

      read(*,*,err=990,end=990) T0
      read(*,*,err=990,end=990) d
      read(*,*,err=990,end=990) nbod
      if (nbod.gt.NBODMAX) then
        write(*,*) "# Error nbod = ", nbod, ".gt. NBODMAX = ",NBODMAX
        stop
      endif

      do i = 1, nbod
        read(*,10,err=990,end=990) file_SKY(i)
      enddo
      read(*,10,err=990,end=990) file_TTV
      do i = 1, nbod
        read(*,10,err=990,end=990) file_RV(i)
      enddo
      read(*,10,err=990,end=990) file_ECL

      read(*,*,err=990,end=990) (Lum(i), i = 1,nbod)
      read(*,*,err=990,end=990) (R_star(i), i = 1,nbod)
      read(*,*,err=990,end=990) (m_min(i), i = 1,nbod)
      read(*,*,err=990,end=990) (m_max(i), i = 1,nbod)
      read(*,*,err=990,end=990) w_SKY, w_TTV, w_RV, w_ECL
      read(*,*,err=990,end=990) eps_BS
      read(*,*,err=990,end=990) debug
      read(*,*,err=990,end=990) debug_swift

c  write input to terminal
      if (debug) then
        write(*,*) "# nparam = ", nparam
        do i = 1,nparam
          write(*,*) "# x_param(", i, ") = ", x_param(i)
        enddo
        write(*,*) "# T0 = ", T0
        write(*,*) "# d = ", d, " pc"
        write(*,*) "# nbod = ", nbod
        do i = 1,nbod
          write(*,*) "# file_SKY(", i, ") = ",
     :      file_SKY(i)(1:length(file_SKY(i)))
        enddo
        write(*,*) "# file_TTV = ", file_TTV(1:length(file_TTV))
        do i = 1,nbod
          write(*,*) "# file_RV(", i, ") = ",
     :      file_RV(i)(1:length(file_RV(i)))
        enddo
        write(*,*) "# file_ECL = ", file_ECL(1:length(file_ECL))
        do i = 1,nbod
          write(*,*) "# Lum(", i, ") = ", Lum(i), " L_S"
        enddo
        do i = 1,nbod
          write(*,*) "# R_star(", i, ") = ", R_star(i), " R_S"
        enddo
        do i = 1,nbod
          write(*,*) "# m_min(", i, ") = ", m_min(i), " M_S"
        enddo
        do i = 1,nbod
          write(*,*) "# m_max(", i, ") = ", m_max(i), " M_S"
        enddo
        write(*,*) "# w_SKY = ", w_SKY
        write(*,*) "# w_TTV = ", w_TTV
        write(*,*) "# w_RV = ", w_RV
        write(*,*) "# w_ECL = ", w_ECL
        write(*,*) "# eps_BS = ", eps_BS
        write(*,*) "# debug = ", debug
        write(*,*) "# debug_swift = ", debug_swift

        open(unit=10,file="T0.plt",status="unknown")
        write(10,*) "T0 = ", T0
        close(10)
      endif

c  calculate chi^2 and the corresponding probability
      ndim = nparam
      do i = 1, nparam
        x(i) = x_param(i)
        variable(i) = .TRUE.
      enddo

      chi2 = chi2_func(x)
      nu = n_fit - nparam
      probp = gammp(nu/2.d0,chi2/2.d0)
      probq = gammq(nu/2.d0,chi2/2.d0)

c  write output
      write(*,*) '# chi2 & probp & probq & ',
     :  'n_SKY & n_TTV & n_RV & n_ECL & ',
     :  'chi2_SKY & chi2_TTV & chi2_RV & chi2_ECL & chi2_MASS'
      write(*,*) chi2, probp, probq,
     :  n_SKY, n_TTV, n_RV, n_ECL,
     :  chi2_SKY, chi2_TTV, chi2_RV, chi2_ECL, chi2_MASS

      stop

c  error handlers
990   continue
      write(*,*) 'chi2: Error reading standard input.'

      end


