c simann.f
c Simmulated annealing for the xi Tau problem.
c Miroslav Broz (miroslav.broz@email.cz), Sep 1st 2015

      program simann

      implicit none
      include '../simplex/simplex.inc'
      include '../simplex/dependent.inc'
      include '../simplex/cb_itmax.inc'

      integer i,j,iter,mp,np,id(NDIMMAX+1)
      real*8 x(NDIMMAX),e(NDIMMAX),p(NDIMMAX+1,NDIMMAX),y(NDIMMAX+1),
     :  ftol,xtry(NDIMMAX),e_param(NDIMMAX)
      integer itertot,iter_at_temp
      real*8 pb(NDIMMAX),yb,temptr,eps_temptr
      character*80 str

      integer length
      real*8 chi2_func

c  read input parameters
5     continue
        read(*,10,err=990,end=990) str
10      format(a)
      if (str(1:1).eq.'#') goto 5

      read(str,*,err=990,end=990) nparam
      if (nparam.gt.NDIMMAX) then
        write(*,*) 'Error: nparam > NDIMMAX = ', NDIMMAX
        stop
      endif

      read(*,*,err=990,end=990) (x_param(i), i=1,nparam)
      read(*,*,err=990,end=990) (e_param(i), i=1,nparam)
      read(*,*,err=990,end=990) (variable(i), i=1,nparam)

      read(*,*,err=990,end=990) T0
      read(*,*,err=990,end=990) d
      read(*,*,err=990,end=990) nbod
      if (nbod.gt.NBODMAX) then
        write(*,*) "# Error nbod = ", nbod, ".gt. NBODMAX = ",NBODMAX
        stop
      endif

      do i = 1, nbod
        read(*,10,err=990,end=990) file_SKY(i)
      enddo
      read(*,10,err=990,end=990) file_TTV
      do i = 1, nbod
        read(*,10,err=990,end=990) file_RV(i)
      enddo
      read(*,10,err=990,end=990) file_ECL

      read(*,*,err=990,end=990) (Lum(i), i = 1,nbod)
      read(*,*,err=990,end=990) (R_star(i), i = 1,nbod)
      read(*,*,err=990,end=990) (m_min(i), i = 1,nbod)
      read(*,*,err=990,end=990) (m_max(i), i = 1,nbod)
      read(*,*,err=990,end=990) w_SKY, w_TTV, w_RV, w_ECL
      read(*,*,err=990,end=990) eps_BS
      read(*,*,err=990,end=990) ftol
      read(*,*,err=990,end=990) ITMAX
      read(*,*,err=990,end=990) iter_at_temp
      read(*,*,err=990,end=990) temptr
      read(*,*,err=990,end=990) eps_temptr
      read(*,*,err=990,end=990) debug
      read(*,*,err=990,end=990) debug_swift

c  write input to terminal
      if (debug) then
        do i = 1,nparam
          write(*,*) "# x_param(", i, ") = ", x_param(i)
        enddo
        do i = 1,nparam
          write(*,*) "# e_param(", i, ") = ", e_param(i)
        enddo
        do i = 1,nparam
          write(*,*) "# variable(", i, ") = ", variable(i)
        enddo
        write(*,*) "# T0 = ", T0
        write(*,*) "# d = ", d, " pc"
        write(*,*) "# nbod = ", nbod
        do i = 1,nbod
          write(*,*) "# file_SKY(", i, ") = ",
     :      file_SKY(i)(1:length(file_SKY(i)))
        enddo
        write(*,*) "# file_TTV = ", file_TTV(1:length(file_TTV))
        do i = 1,nbod
          write(*,*) "# file_RV(", i, ") = ",
     :      file_RV(i)(1:length(file_RV(i)))
        enddo
        write(*,*) "# file_ECL = ", file_ECL(1:length(file_ECL))
        do i = 1,nbod
          write(*,*) "# Lum(", i, ") = ", Lum(i), " L_S"
        enddo
        do i = 1,nbod
          write(*,*) "# R_star(", i, ") = ", R_star(i), " R_S"
        enddo
        do i = 1,nbod
          write(*,*) "# m_min(", i, ") = ", m_min(i), " M_S"
        enddo
        do i = 1,nbod
          write(*,*) "# m_max(", i, ") = ", m_max(i), " M_S"
        enddo
        write(*,*) "# w_SKY = ", w_SKY
        write(*,*) "# w_TTV = ", w_TTV
        write(*,*) "# w_RV = ", w_RV
        write(*,*) "# w_ECL = ", w_ECL
        write(*,*) "# eps_BS = ", eps_BS
        write(*,*) "# ftol = ", ftol
        write(*,*) "# ITMAX = ", ITMAX
        write(*,*) "# iter_at_temp = ", iter_at_temp
        write(*,*) "# temptr = ", temptr
        write(*,*) "# eps_temptr = ", eps_temptr
        write(*,*) "# debug = ", debug
        write(*,*) "# debug_swift = ", debug_swift
      endif

c  resolve variable/fixed parameters
      ndim = 0
      do i = 1,nparam
        if (variable(i)) then
          ndim = ndim + 1
          x(ndim) = x_param(i)
          e(ndim) = e_param(i)
        endif
      enddo

      if (debug) then
        write(*,*) '# ndim = ', ndim
      endif

c  pass the rest in common block /dependent/ 

c  initialise the simplex (p array)

      do i = 1,ndim+1
        do j = 1,ndim
          if (j.eq.i) then
            p(i,j) = x(j) + e(i)
          else
            p(i,j) = x(j)
          endif
        enddo
      enddo

      if (debug) then
        write(*,*) "# initial p() array:"
        do i = 1,ndim+1
          do j = 1,ndim
            write(*,20) p(i,j)
20          format(f16.8,1x,$)
          enddo
          write(*,*)
        enddo
        write(*,*)
      endif

c  y() array
      do i = 1,ndim+1
        do j = 1,ndim
          xtry(j) = p(i,j)
        enddo
        y(i) = chi2_func(xtry)
      enddo

      if (debug) then
        write(*,*) "# initial y() array:"
        do i = 1,ndim+1
          write(*,*) y(i)
        enddo
        write(*,*)
      endif

      mp = NDIMMAX+1
      np = NDIMMAX

      yb = 1.d38
      do i = 1,ndim
        pb(i) = 0.d0
      enddo

c  run it!
      itertot = 0
      iter = 0
      do while ((iter.le.0).and.(itertot.le.ITMAX))
        iter = iter_at_temp
        itertot = itertot + iter

        call amebsa(p,y,mp,np,ndim,pb,yb,ftol,chi2_func,iter,temptr)

        temptr = (1.d0-eps_temptr)*temptr  ! annealing schedule
        itertot = itertot - iter

        if (debug) then
          write(*,*) "# iter = ", iter
          write(*,*) "# itertot = ", itertot
          write(*,*) "# temptr = ", temptr
        endif
      enddo

c  sort the output according to chi^2
      call srtidx(ndim+1,y,id)

c  write the result of minimalisation

      if (debug) then
        write(*,*) "# p() array:"
        do i = 1,ndim+1
          do j = 1,ndim
            write(*,20) p(id(i),j)
          enddo
          write(*,*)
        enddo

        write(*,*) "# y() array:"
        do i = 1,ndim+1
          write(*,*) y(id(i))
        enddo
        write(*,*)
      endif

c      write(*,*) (p(id(1),j), j = 1,ndim), y(id(1))
      write(*,*) (pb(j), j = 1,ndim), yb

      stop

c  error handlers
990   continue
      write(*,*) 'simplex: Error reading standard input.'

      end


